@@ -37,37 +37,35 @@ MAPI_GridHexagonal <- function(samples, crs, hw, buf=0) {
     samples2 <- sf::st_as_sf(samples, coords=c("x", "y"), crs=crs)
     
     if (sf::st_is_longlat(samples2)) {
-            message(sprintf("Suggested specifications for dggridR: spacing=%f, metric=TRUE, resround='down'", hw*2/1000))
-            stop("Package 'dggridR' is no more available from CRAN, therefore it can't be used in a CRAN pacakge such as 'mapi'.\nYou may nevertheless build the grid outside MAPI.\nSee https://github.com/r-barnes/dggridR for more informations.\nFurthermore, some pre-computed grids (levels 1..11) are available from MAPI website.")
-#         # We expect a worldwide dataset
-#         if (base::requireNamespace("dggridR", quietly=TRUE)) {
-#             # WARNING: dggridR is currently not available from CRAN
-#             # For more information see: https://github.com/r-barnes/dggridR
-#             
-#             message("Building worldwide lat/lon grid...")
-#             # Build grid specifications
-#             # NOTE: spacing in km!
-#             my.grid.specs <- dggridR::dgconstruct(spacing=hw*2/1000, metric=TRUE, resround='down')
-#             
-#             # Create grid
-#             grid0 <- dggridR::dgearthgrid(my.grid.specs)
-#             
-#             # Rename grid cell id
-#             colnames(grid0)[1] <- "gid"
-#             
-#             # Convert to sf (tibble -> data.frame)
-#             grid <- sf::st_as_sf(base::as.data.frame(grid0), sf_column_name="geometry")
-#             
-#             # Add centroid coordinates
-#             ctr <- s2::s2_centroid(grid0)
-#             grid$x <- s2::s2_x(ctr)
-#             grid$y <- s2::s2_y(ctr)
-#             
-#             # Reorder columns
-#             grid <- grid[ , c("gid","x","y","geometry")]
-#         } else {
-#             stop("Error: Package 'dggridR' is not installed and is not available from CRAN.\nSee https://github.com/r-barnes/dggridR for more informations.\nSome pre-computed grids (levels 1..11) are available from MAPI website.")
-#         }
+        # We expect a worldwide dataset
+        if (base::requireNamespace("dggridR", quietly=TRUE)) {
+            # WARNING: dggridR is currently not available from CRAN
+            # For more information see: https://github.com/r-barnes/dggridR
+            
+            message("Building worldwide lat/lon grid...")
+            # Build grid specifications
+            # NOTE: spacing in km!
+            my.grid.specs <- dggridR::dgconstruct(spacing=hw*2/1000, metric=TRUE, resround='down')
+            
+            # Create grid
+            grid0 <- dggridR::dgearthgrid(my.grid.specs)
+            
+            # Rename grid cell id
+            colnames(grid0)[1] <- "gid"
+            
+            # Convert to sf (tibble -> data.frame)
+            grid <- sf::st_as_sf(base::as.data.frame(grid0), sf_column_name="geometry")
+            
+            # Add centroid coordinates
+            ctr <- s2::s2_centroid(grid0)
+            grid$x <- s2::s2_x(ctr)
+            grid$y <- s2::s2_y(ctr)
+            
+            # Reorder columns
+            grid <- grid[ , c("gid","x","y","geometry")]
+        } else {
+            stop("Error: Package 'dggridR' is not installed.")
+        }
         
     } else {
     
