@@ -27,12 +27,14 @@
 #' @param dMin minimum distance between individuals. 0 by default.
 #' @param dMax maximal distance between individuals. +Inf by default.
 #' @param use_s2 transform grid and coordinates in latitude,longitude in order to use s2 library functions (default FALSE).
-#'   When set to TRUE MAPI is able to process worldwide grids and ellipses and generate results on the sphere.
+#'   When set to TRUE mapi is able to process worldwide grids and ellipses and generate results on the sphere.
 #' @param nbCores number of CPU cores you want to use during parallel computation. 
 #'   The default value is estimated as the number of available cores minus 1, suitable for a personal computer. 
 #'   On a cluster you might have to set it to a reasonable value (eg. 8) in order to keep resources for other tasks. 
 #' @param N number of points used per quarter of ellipse, 8 by default. 
 #'   Don't change it unless you really know what you are doing.
+#' @param ignore.weights allow to compute a simple mean instead of the default weighted mean.
+#'   This implies that all ellipses bear the same weight.
 #' 
 #' @return a spatial object of class 'sf' providing for each cell: \cr
 #'  - gid: Cell ID \cr
@@ -72,17 +74,19 @@
 #' my.results <- MAPI_RunOnGrid(samples, metric, grid=my.grid, nbPermuts=10, nbCores=1)
 #' 
 #' # eg. Export results to shapefile "myFirstMapiResult" in current directory
-#' library(sf)
-#' st_write(my.results, dsn=".", layer="myFirstMapiResult", driver="ESRI Shapefile")
+#' sf::st_write(my.results, dsn=".", layer="myFirstMapiResult", driver="ESRI Shapefile")
 #' }
 #' 
 
 
-MAPI_RunOnGrid <- function(samples, metric, grid, isMatrix=FALSE, ecc=0.975, errRad=10, nbPermuts=0, dMin=0, dMax=Inf, use_s2=FALSE, nbCores=parallel::detectCores()-1, N=8) {
+MAPI_RunOnGrid <- function(samples, metric, grid, isMatrix=FALSE, ecc=0.975, errRad=10, nbPermuts=0, dMin=0, dMax=Inf, use_s2=FALSE, nbCores=parallel::detectCores()-1, N=8, ignore.weights=FALSE) {
     message("MAPI COMPUTATION STARTED")
     if (!("sf" %in% class(grid))) {stop("grid parameter is not a 'sf' object!")}
-    if (! base::requireNamespace("doParallel", quietly=TRUE) || ! base::requireNamespace("foreach", quietly=TRUE)) {
+    if (base::requireNamespace("doParallel", quietly=TRUE) && base::requireNamespace("foreach", quietly=TRUE)) {
+        `%dopar%` <- foreach::`%dopar%`
+    } else {
         nbCores <- 1
+        `%dopar%` <- NA
     }
     tot <- system.time({
         my.data.in <- MAPI_CheckData(samples, metric, isMatrix=isMatrix)
@@ -105,7 +109,7 @@ MAPI_RunOnGrid <- function(samples, metric, grid, isMatrix=FALSE, ecc=0.975, err
         
         # force using s2 functions
         if (use_s2) {
-            crs <- 4326
+            crs <- "EPSG:4326"
             my.samples.geom <- sf::st_transform(my.samples.geom, crs)
             my.samples.geom$x <- as.numeric((sf::st_coordinates(my.samples.geom)[,1]))
             my.samples.geom$y <- as.numeric((sf::st_coordinates(my.samples.geom)[,2]))
@@ -161,7 +165,7 @@ MAPI_RunOnGrid <- function(samples, metric, grid, isMatrix=FALSE, ecc=0.975, err
                             #   earth, 
                             #   1st sphere based on first focus and radius from first great circle distance,
                             #   2nd sphere based on second focus and radius from second great circle distance
-                            tp.cpp <- trilaterate_cpp(p0x, p0y, p1x, p1y, gc0, gc1)
+                            tp.cpp <- .trilaterate_cpp(p0x, p0y, p1x, p1y, gc0, gc1)
 
                             # get coordinates for the two points
                             tp1.cpp <- tp.cpp[1:2]
@@ -228,10 +232,13 @@ MAPI_RunOnGrid <- function(samples, metric, grid, isMatrix=FALSE, ecc=0.975, err
             if (mapi.spherical) {
                 if (nbCores > 1) {
                     doParallel::registerDoParallel(nbCores)
-                    ellipses$dist <- foreach::foreach(i=1:nrow(ellipses), .combine=c) %dopar% {
+                    ellipses$dist <- foreach::foreach(i=1:nrow(ellipses), .combine=base::c) %dopar% {
                         i1 <- ellipses$dlid1[i]
                         i2 <- ellipses$dlid2[i]
-                        s2::s2_distance(location.pts[i1], location.pts[i2])
+                        p1 <- location.pts[i1]
+                        p2 <- location.pts[i2]
+                        d <- s2::s2_distance(p1, p2)
+                        return(d)
                     }
                     doParallel::stopImplicitCluster()
                 } else {
@@ -265,8 +272,7 @@ MAPI_RunOnGrid <- function(samples, metric, grid, isMatrix=FALSE, ecc=0.975, err
                 # build ellipses on sphere using companion functions
                 if (nbCores > 1) { 
                     # parallel computation
-#                     doParallel::registerDoParallel(nbCores)
-                    
+                    doParallel::registerDoParallel(nbCores)
                     doIt <- function(todo) {
                         foreach::foreach(i=todo, .export=c("mkEll_s2"), .combine=c) %dopar% {
                             # return as text as pointers vanishes from cluster
@@ -274,7 +280,7 @@ MAPI_RunOnGrid <- function(samples, metric, grid, isMatrix=FALSE, ecc=0.975, err
                         }
                     }
                     ePolysT <- doIt(1:nrow(ellipses.matrix))
-#                     doParallel::stopImplicitCluster()
+                    doParallel::stopImplicitCluster()
                     ePolys <- s2::as_s2_geography(ePolysT) # Therefore, we converts back to s2 geography
                 } else { 
                     # serial computation
@@ -287,13 +293,14 @@ MAPI_RunOnGrid <- function(samples, metric, grid, isMatrix=FALSE, ecc=0.975, err
             } else {
                 # get distinct locations: x,y coordinates and error radius of each point
                 ellipses.matrix <- as.matrix(ellipses[,c("x1", "y1", "x2", "y2", "errRad1", "errRad2")])
-                # call to MAPI c++ function mkP4st_cpp for building the polygon (convex hull of ellipse and error circles)
+                # call to MAPI c++ function .mkP4st_cpp for building the polygon (convex hull of ellipse and error circles)
                 ePolys <- apply(ellipses.matrix, 1, function(r) {
-                    sf::st_polygon(list(mkP4st_cpp(r, N, ecc)))
+                    sf::st_polygon(list(.mkP4st_cpp(r, N, ecc)))
                 })
             }
             # simplify ellipses table
             ellipses <- ellipses[,c("rowid", "loc1", "loc2", "dist")]
+            # compute weight
             # set geometry to ellipses table (same order !)
             if (mapi.spherical) {
                 # compute area
@@ -304,8 +311,11 @@ MAPI_RunOnGrid <- function(samples, metric, grid, isMatrix=FALSE, ecc=0.975, err
                 # compute area
                 ellipses$area <- sf::st_area(ellipses$geometry) # area in "units"
             }
-            # compute weight
-            ellipses$weight <- 1.0 / as.numeric(ellipses$area)
+            if (ignore.weights) {
+                ellipses$weight <- 1.0
+            } else {
+                ellipses$weight <- 1.0 / as.numeric(ellipses$area)
+            }
             
             # free memory
             rm(ellipses.matrix)
@@ -395,7 +405,7 @@ MAPI_RunOnGrid <- function(samples, metric, grid, isMatrix=FALSE, ecc=0.975, err
                 return(v)
             })
             # Fast count
-            nbMatches <- countMatches_cpp(inter)
+            nbMatches <- .countMatches_cpp(inter)
         })
         tv <- as.vector(t) ; tv[is.na(tv)] <- 0.0
         if (is.na(nbMatches)) { nbMatches <- 0 }
@@ -408,7 +418,7 @@ MAPI_RunOnGrid <- function(samples, metric, grid, isMatrix=FALSE, ecc=0.975, err
         message("Computing values for grid cells...")
         t <- system.time({
             # Call to C++ function parseInter_cpp which iterates on grid, intersetion and sample pairs for computing values
-            resu <- data.table::as.data.table(parseInter_cpp(grid$gid, inter, sampX$weight, sampX$value))
+            resu <- data.table::as.data.table(.parseInter_cpp(grid$gid, inter, sampX$weight, sampX$value))
             colnames(resu) <- c("gid", "nb_ell", "avg_value", "sum_wgts", "w_stdev")
             data.table::setkey(resu, "gid")
             # computes sum-of-weights percentile
@@ -439,7 +449,7 @@ MAPI_RunOnGrid <- function(samples, metric, grid, isMatrix=FALSE, ecc=0.975, err
                     doIt <- function(todo) {
                         resu2 <- foreach::foreach( i=todo, .verbose=FALSE, .combine=cbind ) %dopar% {
                             cat(sprintf("%d ",i))
-                            doPerm(i, my.samples_perm, my.samples, sampX, my.metric, inter, grid)
+                            .doPerm(i, my.samples_perm, my.samples, sampX, my.metric, inter, grid)
                         }
                     }
                     resu2 <- doIt(1:nbPermuts)
@@ -450,7 +460,7 @@ MAPI_RunOnGrid <- function(samples, metric, grid, isMatrix=FALSE, ecc=0.975, err
                     message(sprintf("... serially ..."))
                     pbs <- utils::txtProgressBar(max=nbPermuts, style=3, width=80)
                     resu2 <- data.table::as.data.table(lapply(1:nbPermuts, function(i){
-                        pp <- doPerm(i, my.samples_perm, my.samples, sampX, my.metric, inter, grid)
+                        pp <- .doPerm(i, my.samples_perm, my.samples, sampX, my.metric, inter, grid)
                         utils::setTxtProgressBar(pbs, i)
                         return(pp)
                     }))
@@ -483,7 +493,6 @@ MAPI_RunOnGrid <- function(samples, metric, grid, isMatrix=FALSE, ecc=0.975, err
     }
     
     tv <- as.vector(tot) ; tv[is.na(tv)] <- 0.0
-    # resu$permuts <- NULL # discard permutations due to export problems DEPRECATED as the column is not created anymore
     message(sprintf("MAPI COMPUTATION ENDED.  [TOTAL: user: %0.3f, system: %0.3f, elapsed: %0.3f seconds]", tv[1]+tv[4], tv[2]+tv[5], tv[3]))
     return(resu)
 }
@@ -492,19 +501,20 @@ MAPI_RunOnGrid <- function(samples, metric, grid, isMatrix=FALSE, ecc=0.975, err
 
 #### Internal function
 
+#' @noRd
 # prepare permutation function
-doPerm <- function(permut, my.samples_perm, my.samples, sampX, my.metric, inter, grid) {
-    # shuffle individuals 
+.doPerm <- function(permut, my.samples_perm, my.samples, sampX, my.metric, inter, grid) {
+    # shuffle individuals
     my.samples_perm$indP <- base::sample(my.samples$ind)
-    
+
     # replace unpermuted individuals by permuted ones
     sampX.P <- data.table::merge.data.table(sampX,   my.samples_perm, by.x="ind1", by.y="ind")
     sampX.P <- data.table::merge.data.table(sampX.P, my.samples_perm, by.x="ind2", by.y="ind", suffixes=c("1","2"))
-    
+
     # merge permuted samplepairs with metric
     sampX.P <- data.table::merge.data.table(sampX.P, my.metric, all.x=TRUE, by.x=c("indP1","indP2"), by.y=c("ind1","ind2"))
     data.table::setkey(sampX.P, "id")
-    
+
     # compute permuted result
-    return( parseInterPerm_cpp(grid$gid, inter, sampX.P$weight, sampX.P$value.y) )
+    return( .parseInterPerm_cpp(grid$gid, inter, sampX.P$weight, sampX.P$value.y) )
 }
